@@ -32,7 +32,7 @@
  */
 
 /* 
- * $Id: kadmin_locl.h,v 1.45 2004/06/27 15:04:07 joda Exp $
+ * $Id: kadmin_locl.h 17580 2006-05-13 21:28:56Z lha $
  */
 
 #ifndef __ADMIN_LOCL_H__
@@ -90,6 +90,7 @@
 #include <krb5_locl.h>
 #include <hdb.h>
 #include <hdb_err.h>
+#include <hex.h>
 #include <kadm5/admin.h>
 #include <kadm5/private.h>
 #include <kadm5/kadm5_err.h>
@@ -104,43 +105,32 @@ extern void * kadm_handle;
 
 /* util.c */
 
-void attributes2str(krb5_flags attributes, char *str, size_t len);
-int  str2attributes(const char *str, krb5_flags *flags);
-int  parse_attributes (const char *resp, krb5_flags *attr, int *mask, int bit);
-int  edit_attributes (const char *prompt, krb5_flags *attr, int *mask,
-		      int bit);
+void attributes2str(krb5_flags, char *, size_t);
+int  str2attributes(const char *, krb5_flags *);
+int  parse_attributes (const char *, krb5_flags *, int *, int);
+int  edit_attributes (const char *, krb5_flags *, int *, int);
 
-void time_t2str(time_t t, char *str, size_t len, int include_time);
-int  str2time_t (const char *str, time_t *time);
-int  parse_timet (const char *resp, krb5_timestamp *value, int *mask, int bit);
-int  edit_timet (const char *prompt, krb5_timestamp *value, int *mask,
-		 int bit);
+void time_t2str(time_t, char *, size_t, int);
+int  str2time_t (const char *, time_t *);
+int  parse_timet (const char *, krb5_timestamp *, int *, int);
+int  edit_timet (const char *, krb5_timestamp *, int *,
+		 int);
 
-void deltat2str(unsigned t, char *str, size_t len);
-int  str2deltat(const char *str, krb5_deltat *delta);
-int  parse_deltat (const char *resp, krb5_deltat *value, int *mask, int bit);
-int  edit_deltat (const char *prompt, krb5_deltat *value, int *mask, int bit);
+void deltat2str(unsigned, char *, size_t);
+int  str2deltat(const char *, krb5_deltat *);
+int  parse_deltat (const char *, krb5_deltat *, int *, int);
+int  edit_deltat (const char *, krb5_deltat *, int *, int);
 
-int edit_entry(kadm5_principal_ent_t ent, int *mask,
-	       kadm5_principal_ent_t default_ent, int default_mask);
-void set_defaults(kadm5_principal_ent_t ent, int *mask,
-		  kadm5_principal_ent_t default_ent, int default_mask);
-int set_entry(krb5_context context,
-	      kadm5_principal_ent_t ent,
-	      int *mask,
-	      const char *max_ticket_life,
-	      const char *max_renewable_life,
-	      const char *expiration,
-	      const char *pw_expiration,
-	      const char *attributes);
+int edit_entry(kadm5_principal_ent_t, int *, kadm5_principal_ent_t, int);
+void set_defaults(kadm5_principal_ent_t, int *, kadm5_principal_ent_t, int);
+int set_entry(krb5_context, kadm5_principal_ent_t, int *,
+	      const char *, const char *, const char *,
+	      const char *, const char *);
 int
-foreach_principal(const char *exp, 
-		  int (*func)(krb5_principal, void*), 
-		  const char *funcname,
-		  void *data);
+foreach_principal(const char *, int (*)(krb5_principal, void*), 
+		  const char *, void *);
 
-int parse_des_key (const char *key_string,
-		   krb5_key_data *key_data, const char **err);
+int parse_des_key (const char *, krb5_key_data *, const char **);
 
 /* server.c */
 
@@ -150,7 +140,7 @@ kadmind_loop (krb5_context, krb5_auth_context, krb5_keytab, int);
 /* random_password.c */
 
 void
-random_password(char *pw, size_t len);
+random_password(char *, size_t);
 
 /* kadm_conn.c */
 
