@@ -33,13 +33,13 @@
 
 #include "hprop.h"
 
-RCSID("$Id: hpropd.c 20434 2007-04-19 14:00:32Z lha $");
+RCSID("$Id: hpropd.c 22245 2007-12-08 23:48:52Z lha $");
 
 static int inetd_flag = -1;
 static int help_flag;
 static int version_flag;
 static int print_dump;
-static const char *database = HDB_DEFAULT_DB;
+static const char *database;
 static int from_stdin;
 static char *local_realm;
 static char *ktname = NULL;
@@ -111,6 +111,9 @@ main(int argc, char **argv)
     if (argc != 0)
 	usage(1);
 
+    if (database == NULL)
+	database = hdb_default_db(context);
+
     if(from_stdin)
 	fd = STDIN_FILENO;
     else {
