@@ -33,13 +33,13 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: ewrite.c 14773 2005-04-12 11:29:18Z lha $");
+RCSID("$Id: ewrite.c 21005 2007-06-08 01:54:35Z lha $");
 #endif
 
 #include <unistd.h>
 #include <err.h>
 
-#include <roken.h>
+#include "roken.h"
 
 /*
  * Like write but never fails (and never returns partial data).
